/**
 *
 * Copyright (C) 2000 Vandoorselaere Yoann, Thomas Mirlacher
 * All Rights Reserved
 *
 * This file is part of the Mandrake OMS program.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/


#define __XML_PARSE_C__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "xmlparse.h"

static void _oms_xml_parse_section (xmlDocPtr doc, xmlNodePtr cur);
static xmlptr_t _oms_xml_handle_tag (xmlDocPtr doc, xmlNodePtr cur);

static oms_xmltree_t *xhead = NULL;
static xmlhandler_tbl_t *xmlhandler_tbl;


/**
 * Search the current tag in our current tag_tbl table.
 **/

static inline xmlptr_t _oms_xml_handle_tag (xmlDocPtr doc, xmlNodePtr cur)
{
	int i;

	if (!cur || !cur->name)
		return NULL;

	for (i=0; xmlhandler_tbl[i].name; i++) {
		if (!strcmp (cur->name, xmlhandler_tbl[i].name))
			return xmlhandler_tbl[i].handler;
	}

	return NULL;
}


/**
 * This one should be left public cause it is succeptible to
 * be called back by some tag in xmltag_*.c, this is a recursive function.
 * (it will probably really be used when the OMS xml file format will be
 * well defined).
 **/

static inline void _oms_xml_parse_section (xmlDocPtr doc, xmlNodePtr cur)
{
        for (cur=cur->childs; cur; cur=cur->next ) {
        	xmlptr_t handler;
       
                if (!(handler = _oms_xml_handle_tag (doc, cur))) {
                        fprintf (stderr, "Tag %s doesn't have a function handler.\n", cur->name);
                        continue;
                }

                handler (doc, cur);
        }
}


/**
 *
 **/

int oms_xml_init (xmlhandler_tbl_t *_xmlhandler_tbl)
{
	if (_xmlhandler_tbl) {
		xmlhandler_tbl = _xmlhandler_tbl;
		return 0;
	}

	return -1;
}


/**
 *
 **/

oms_xmltree_t *oms_xml_list_add (void *data)
{
	static oms_xmltree_t *xtail;
	oms_xmltree_t *item;

        if (!(item = (oms_xmltree_t *) calloc (1, sizeof (oms_xmltree_node_t)))) {
                perror ("calloc");
		return NULL;
        }

        if (!xhead) {
                xhead = item;
                xtail = item;
        } else {
                xtail->next = item;
                item->prev = xtail;
                xtail = item;
        }

	item->data = data;

	return item;
}


/**
 * Take the XML file to parse as argument.
 **/

extern int xmlDoValidityCheckingDefaultValue;

oms_xmltree_t *oms_xml_parse_file (const char *file) 
{
        xmlDocPtr doc;
        xmlNodePtr cur;

	xmlDoValidityCheckingDefaultValue = 1;

        if (!(doc = xmlParseFile (file))) {
                perror ("xmlParseFile");
                return NULL;
        }

        if (!(cur = doc->root)) {
                fprintf (stderr, "%s seems to be empty.\n", file);
                return NULL;
        }
       
        _oms_xml_parse_section (doc, cur);

        return xhead;
}
