/*****
*
* This file is part of the OMS program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

struct Rtcp_Common_Header {
#if BYTE_ORDER == LITTLE_ENDIAN
	uint8_t count		:5;	// varies by packet type
	uint8_t padding		:1;	// padding flag
	uint8_t version		:2;	// protocol version
#elif BYTE_ORDER == BIG_ENDIAN
	uint8_t version		:2;	// protocol version
	uint8_t padding		:1;	// padding flag
	uint8_t count		:5;	// varies by packet type
#endif
	uint8_t packet_type	:8;	// RTCP packet type
	uint16_t length;		// pkt len in words, w/o this word
};

struct Rtcp_RR {
	uint32_t ssrc;			// data source being reported
	unsigned int fraction:8;	// fraction lost since last SR/RR
	int lost:24;			// cumul. no. pkts lost (signed!)
	uint32_t last_seq;		// extended last seq. no. received
	uint32_t jitter;		// interarrival jitter
	uint32_t lsr;			// last SR packet from this source
	uint32_t dlsr;			// delay since last SR packet
};

struct Rtcp_SR_Header {
	uint32_t ssrc;			// sender generating this report
	uint32_t ntp_sec;		// NTP timestamp
	uint32_t ntp_frac;
	uint32_t rtp_ts;		// RTP timestamp
	uint32_t psent;			// packets sent
	uint32_t osent;			// octets sent
	struct Rtcp_RR rr[1];		// variable-length list
};
struct Rtcp_RR_Header {
	uint32_t ssrc;			// receiver generating this report
	struct Rtcp_RR rr[1];		// variable-length list
};

struct SDES_Item {
	uint8_t type;			// type of item (Rtcp_Sdes_Type)
	uint8_t length;			// length of item (in octets)
	char data[1];			// text, not null-terminated
};

struct Rtcp_SDES_Header {
	uint32_t src;			// first SSRC/CSRC
	struct SDES_Item item[1];	// list of SDES items
};

struct Rtcp_BYE_Header {
	uint32_t src[1];		// list of sources
	char data[1];			// reason for leaving
};

struct Rtcp_APP_Header {
	uint32_t ssrc;			// source
	char name[4];			// name
	char data[1];			// application data
};
