/*****
*
* This file is part of the OMS program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>

#include <oms/oms.h>
#include <oms/buf.h>
#include <oms/plugin/codec.h>

#include "mgadvd.h"

static int _mgadvd_init (plugin_codec_t *plugin, buf_t *buf, char *output_path);
//static int _mgadvd_open (plugin_codec_t *plugin);
static int _mgadvd_read (plugin_codec_t *plugin, buf_t *buf, buf_entry_t *buf_entry);
static int _mgadvd_set_stat (plugin_codec_t *plugin, uint status);
static int _mgadvd_set_attr (plugin_codec_t *plugin, uint id, void *attr);

static struct {
	int fd_mgadvd;
	int current_audio_mode;
	int current_audio_stream;
	buf_t *buf;
	int status;
} decoder;

static plugin_codec_t codec_mgadvd = {
	&decoder,
	_mgadvd_init,
	NULL,		// exit
        NULL,		// open
        NULL,		// close
	_mgadvd_read,	// read
	NULL,		// write
	_mgadvd_set_stat,
	_mgadvd_set_attr,
};


/**
 *
 **/

static int _mgadvd_read (plugin_codec_t *plugin, buf_t *buf, buf_entry_t *buf_entry)
{
	uint size = buf_entry->mem_len;

	while (size)
		size -= write (decoder.fd_mgadvd, buf_entry->mem, size);

	return 0;
}


/**
 *
 **/

static int _mgadvd_init (plugin_codec_t *plugin, buf_t *buf, char *output_path) {
	fprintf(stderr,"DecoderInit\n");

  // open device
	if ((decoder.fd_mgadvd = open(output_path, O_WRONLY)) < 0) {
		perror (output_path);
		return -1;
	}  

	decoder.buf = buf;
	decoder.current_audio_mode = -1;
	decoder.current_audio_stream = -1;

	return 0;
}


/**
 *
 **/

static int _mgadvd_set_attr (plugin_codec_t *plugin, uint id, void *attr)
{
	return 0;
}


/**
 *
 **/

static int _mgadvd_set_stat (plugin_codec_t *plugin, uint status)
{
  	switch (status) {
		case STATUS_PLAY:
			ioctl(decoder.fd_mgadvd, I34_IOCTSTART);
			break;
		case STATUS_STOP:
			fprintf(stderr,"DecoderStop\n");
	
			if (ioctl(decoder.fd_mgadvd, I34_IOCTSTOP))
				return -1;

			break;
		case STATUS_PAUSE:
			break;
		default:
			fprintf (stderr, "DecoderSetStatus: unhandled status: %d\n", status);
			return -1;
	}

	return 0;
}


/**
 * Initialize Plugin.
 **/

int plugin_init (char *whoami)
{
	pluginRegister (whoami,
		PLUGIN_ID_CODEC_VIDEO,
		STREAM_ID_MPEG2,
		&codec_mgadvd);

	return 0;
}


/**
 * Cleanup Plugin.
 **/

void plugin_exit (void)
{
}
