/*****
*
* This file is part of the OMS program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

//PLUGIN_INFO(INFO_NAME, "Null audio driver output");
//PLUGIN_INFO(INFO_AUTHOR, "Aaron Holtzman <aholtzma@ess.engr.uvic.ca>");

/*
 *
 *  audio_esd.c
 *    
 *	Copyright (C) Thomas Mirlacher
 *
 * GPL ...
 *
 */

#include <unistd.h>
#include <esd.h>
#include <oms/log.h>
#include <oms/plugin/output_audio.h>

static int _audio_esd_open (void *plugin, void *name);
static int _audio_esd_close (void *plugin);
static int _audio_esd_setup (plugin_output_audio_attr_t * attr);
static int _audio_esd_write (const void *buf, size_t num_bytes);

struct {
	int fd;
	char *server;
} _audio_esd_priv = {
fd: -1, server:0,};

static plugin_output_audio_t audio_esd = {
	open:_audio_esd_open,
	close:_audio_esd_close,
	setup:_audio_esd_setup,
	write:_audio_esd_write config:NULL,
};

static int _audio_esd_open (void *plugin, void *name)
{
	return 0;
}

static int _audio_esd_close (void *plugin)
{
	if (_audio_esd_priv.fd >= 0)
		esd_close (_audio_esd_priv.fd);

	_audio_esd_priv.fd = -1;

	return 0;
}

static int _audio_esd_setup (plugin_output_audio_attr_t * attr)
{
	esd_format_t format = ESD_STREAM | ESD_PLAY;

	format |= ESD_BITS16 | ESD_STEREO;

	_audio_esd_priv.fd =
	    esd_play_stream_fallback (format, attr->speed,
				      _audio_esd_priv.server, "oms");

	return 0;
}

static int _audio_esd_write (const void *buf, size_t num_bytes)
{
	write (_audio_esd_priv.fd, buf, num_bytes);
	return 0;
}

int plugin_init (char *whoami)
{
	pluginRegister (whoami,
		PLUGIN_ID_OUTPUT_AUDIO,
		NULL,
		"esd ",
		NULL,
		&audio_esd);

	return 0;
}

void plugin_exit (void)
{
}
