/*****
*
* This file is part of the OMS program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

/*
  definitions for video library
  Copyright (C) 2000  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */


#ifndef __OMS__

// ---------- <USE FROM OMS> ----------

# define PLUGIN_INIT(x)  x ## _init
# define PLUGIN_EXIT(x)  x ## _exit

#define IMG_ALLOC(h,w,f)                                                \
        frame_t *frame;                                                 \
                                                                        \
        if (!(frame = malloc (sizeof (frame_t))))                       \
                return NULL;                                            \
                                                                        \
        /* we only know how to do 4:2:0 planar yuv right now. */        \
        if (!(frame->private = malloc (width * height * 3 / 2))) {      \
                free(frame);                                            \
                return NULL;                                            \
        }                                                               \
									\
	frame->base[0] = frame->private;				\
	frame->base[1] = frame->base[0] + width * height;		\
	frame->base[2] = frame->base[0] + width * height * 5 / 4;	\
                                                                        \
        return frame


#define IMG_FREE(frame)                                                 \
        free (frame->private);                                          \
        free (frame)



//
// We use our own structures
//
typedef struct vo_info_s
{
        /* driver name ("Matrox Millennium G200/G400" */
        const char *name;
        /* short name (for config strings) ("mga") */
        const char *short_name;
        /* author ("Aaron Holtzman <aholtzma@ess.engr.uvic.ca>") */
        const char *author;
        /* any additional comments */
        const char *comment;
} vo_info_t;

typedef struct overlay_buf_struct {
        unsigned char *data;    // 7-4: mixer key, 3-0: color index
        unsigned int x;         // x start of subpicture area
        unsigned int y;         // y start of subpicture area
        int width;              // width of subpicture area
        int height;             // height of subpicture area

        unsigned char clut[4];  // color lookup table
        unsigned char trans[4]; // mixer key table

        unsigned int time_execute;     // time in ms
} overlay_buf_t;

typedef struct frame_s {
	uint8_t *base[3];	// pointer to 3 planes
	void *private;
} frame_t;

typedef struct plugin_output_video_attr_struct {
	int width;
	int height;
	uint32_t fullscreen;
	char *title;
} plugin_output_video_attr_t;

typedef struct plugin_output_video_struct {
        int (* close)            (void * plugin);

	int (* setup)		(plugin_output_video_attr_t * attr);

	int  (* draw_frame)	(frame_t * frame);
	int  (* draw_slice)	(uint8_t * src[], int slice_num);
	void (* flip_page)	(void);
	frame_t *	(* allocate_image_buffer) (int width, int height,
							 uint32_t format);
	void (* free_image_buffer) (frame_t * image);
        // 
        // NOT part in OMS
        //
        const vo_info_t * vo_info;

        int (* overlay)          (overlay_buf_t * overlay_buf, int id);

} plugin_output_video_t;
// ---------- </USE FROM OMS> ----------

// NULL terminated array of all drivers 
extern plugin_output_video_t* video_out_drivers[];

//
// This macro creates a static variable and initialize it
// with given functions
//


#define LIBVIDEO_EXTERN(x,name) \
static plugin_info_t x ## _vo_info = {\
        "",\
        name,\
        "",\
        ""\
};\
plugin_output_video_t video_out_##x = {\
        open:                   x ## _open,\
        close:                  x ## _close,\
	setup:                  x ## _setup,\
	draw_frame:             x ## _draw_frame,\
	draw_slice:             x ## _draw_slice,\
	flip_page:              x ## _flip_page,\
	allocate_image_buffer:  x ## _allocate_image_buffer,\
	free_image_buffer:      x ## _free_image_buffer,\
        vo_info:                &(x ## _vo_info),\
        overlay:                x ## _overlay\
}


#endif
// != __OMS__
